package logic;

import java.util.*;
public class Action {
    
	/** object on which some action must be performed */
	protected Updateable ud;

	/** 
	 *  When rules are executed, this a table of update values
	 *  for the Updateable object is obtained and set here
	 */
	protected Hashtable updateVals;

    /**
     *  This is an attribute that is passed in and set with
     *  the "typical" constructor, following the default
     *  framework behavior. When this constructor is used
     *  the default behavior involves setting this attribute
     *  in the Updateable object
     */
	protected Object attribute;

    /**
     *  This String is used as a key in the default behavior of
     *  this class -- when the first constructor is used,
     *  attribute is set as value in a hashtable, using
     *  MAIN_KEY as key. The hashtable is passed to the Updateable
     *  instance and is unpacked using MAIN_KEY again.
     */
	public static final String MAIN_KEY = "mainKey";

    /**
     *  This is the "default" constructor -- it handles the
     *  simple case in which the required action simply sets
     *  a value in the Updateable object.
     */
	public Action(Updateable ud, Object attribute) {
		this.ud = ud;
		setAttribute(attribute);
	}
	
	/**
	 *  This constructor handles all the more complicated 
	 *  scenarios in which more is required from an action
	 *  then setting a single attribute. When this constructor
	 *  is used, it's necessary to override the execute() method
	 */
	public Action(Updateable ud, Hashtable attributes) {
		this.ud = ud;
		setAttributesFromTable(attributes);
	}
	/**
	 *  What is provided here is default behavior.
	 *  If the Action(Updateable, Object) constructor
	 *  was used, then the action to be performed
	 *  is simply to set a single attribute in the
	 *  Updateable object. The default behavior for
	 *  execute() given below does this, using MAIN_KEY
	 *  as key and the already set ivar attribute as the
	 *  value. If you need a more complicated action,
	 *  this method needs to be overridden.
	 */
	public void execute() {
		Hashtable ht = new Hashtable();
		ht.put(MAIN_KEY, attribute);
		getUd().doUpdate(ht);
	}
	public Updateable getUd() {
		return ud;
	}
	public Hashtable getUpdateVals() {
		return updateVals;
	}
	public void setAttribute(Object ob) {
		attribute = ob;
	}
	public void setAttributesFromTable(Hashtable ht) {}
}