package logic;

import java.util.*;

/**
 *  This class encapsulates the notion of a set of rules that
 *  match a sequence of booleans with an action.
 *  The underlying container of these match-ups is a Hashtable.
 *  The Rules class provides an interface to this underlying
 *  data structure with addRule(..) and lookUp(..)
 */
public class Rules {
	private Hashtable ruleTable = new Hashtable();
	
	public Rules() {}
	
	/** 
	 *  This method adds a new boolean-sequence / action pair
	 *  to the rules Hashtable. It first converts the String boolStr,
	 *  which is a string of T's and F's, to a BooleanTuple, then
	 *  it loads this BooleanTuple as a key in ruleTable, matched
	 *  with the action argument passed in.
	 */
	public void addRule(String boolStr, Action action) throws NestingTooDeepException{
		BooleanTuple [] tuples = BooleanTupleFactory.getTuples(boolStr);
		for(int i = 0; i < tuples.length; ++i) {
			ruleTable.put(tuples[i], action);
		}
	}
	
	/**
	 *  This method looks up, in ruleTable, 
	 *  the action that corresponds to the BooleanTuple
	 *  passed in.
	 */
	public Action lookUp(BooleanTuple bt)throws RuleNotFoundException {
	    Object ob = ruleTable.get(bt);
	    if(ob == null) {
	        throw new RuleNotFoundException(bt.toString());
	    }
	    return (Action)ob;
	}
}